#include <stdio.h>
#include <string.h>
#include <math.h>

char *stridx(char *inp, char idx);
char *strnidx(char *inp, char idx);
void read_values(FILE *inp, FILE *out, int npts, int nvals);
void read_pspice(FILE *inp, FILE *out, int nvals);

char getState(double v);

class Signal
{
public:
	double value;
	double prev;
	double pts[3];
	char state;
	int test(Signal &time);
	double interp(Signal &time, double v);
	void write(FILE *out, int idx);
};

const int BUFLEN = 1024;
char ibuf[BUFLEN];

int main(int argc, char *argv[])
{
	char *filename;
	char *cp;
	FILE *in, *out;

	int nvals = 0;

	filename = argc>1? argv[1]: "spice.txt";
	in = fopen(filename,"r");
	if (!in) {
		printf("File not found: %s\n",filename);
		return -1;
	}

	if (argc>2) {
		filename = argv[2];
		out = fopen(filename,"wt");
	}
	else out = stdout;

	// check for titles
	fgets(ibuf,BUFLEN-1,in);
	cp = ibuf;
	nvals = 0;
	while (cp) {
		nvals++;
		cp = stridx(cp,'\t');
	}
	printf("%d columns\n",nvals);
	

	if (nvals) read_pspice(in, out, nvals);

	fclose(in);
	return 0;
}


void read_pspice(FILE *inp, FILE *out, int nvals)
{
	int i, n=0;
	char *cp, *first;
	double *values;
	Signal *sig;
	values = new double[nvals];
	sig = new Signal[nvals];
	while (1) {
		fgets(ibuf,BUFLEN-1,inp);
		if (feof(inp)) break;
		// strip newline
		stridx(ibuf,'\n');
		first = strnidx(ibuf,' ');
		if (first==0) break;
		// parse numeric values
		for (i=0; i<nvals; i++) {
			cp = stridx(first,'\t');
			sscanf(first,"%lf",&values[i]);
			first = strnidx(cp,' ');
			if (first) continue;
		}
		//printf("%3d\t",n);
		//for (i=0; i<nvals; i++) printf("\t%g",values[i]);
		//printf("\n");
		for (i=0; i<nvals; i++) sig[i].value = values[i];
		if (n==0) {
			// determine state
			for (i=1; i<nvals; i++) {
				sig[i].state = getState(values[i]);
			}
		}
		else {
			for (i=1; i<nvals; i++) {
				if (sig[i].test(sig[0])) sig[i].write(out,(nvals>2? i: 0));
			}
		}
		for (i=0; i<nvals; i++) sig[i].prev = sig[i].value;
		n++;
	}
	delete [] values;
	delete [] sig;
}

int Signal::test(Signal &time)
{
	switch (state) {
	case 'L':
		if (prev<0.5 && value>=0.5) pts[0] = interp(time,0.5);
		if (prev<2.5 && value>=2.5) pts[1] = interp(time,2.5);
		if (prev<4.5 && value>=4.5) {
			pts[2] = interp(time,4.5);
			state = 'H';
			return 1;
		}
		break;
	case 'H':
		if (prev>4.5 && value<=4.5) pts[0] = interp(time,4.5);
		if (prev>2.5 && value<=2.5) pts[1] = interp(time,2.5);
		if (prev>0.5 && value<=0.5) {
			pts[2] = interp(time,0.5);
			state = 'L';
			return 1;
		}
		break;
	case 'X':
		state = getState(value);
		break;
	}
	return 0;
}

double Signal::interp(Signal &time, double v)
{
	if (value==v) return time.value;
	if (prev==v) return time.prev;
	double ds = (value-prev);
	double dt = (time.value - time.prev);
	return time.prev + dt*(v-prev)/ds;
}

void Signal::write(FILE *out, int idx)
{
	int i;
	if (idx) fprintf(out,"%d\t",idx);
	fprintf(out,"%s",state=='H'? "L to H": "H to L");
	for (i=0; i<3; i++) fprintf(out,"\t%g",pts[i]*1e12);
	fprintf(out,"\t%g\n",1.25*(pts[2]-pts[0])*1e12);
}
	

char getState(double v)
{
	if (v<0.5) return 'L';
	else if (v>4.5) return 'H';
	else return 'X';
}

// skip to just past specified character
char *strnidx(char *inp, char idx)
{
	char *out=inp;
	if (!out) return 0;
	while (*out && *out==idx) out++;
	if (*out==0) out=0;
	return out;
}

// replace specified character by null and skip to just past that character
char *stridx(char *inp, char idx)
{
	char *out = inp;
	if (!out) return 0;
	while (*out && *out!=idx) out++;
	if (*out) *out++ = 0;
	else out = 0;
	return out;
}
		
/*
Title: Level 2 NMOS simulation
Date: Thu Sep 14 17:49:22  2000
Plotname: DC transfer characteristic
Flags: real
			   No. Variables: 2
			   No. Points: 404
Variables:
			   0	sweep	voltage
					   1	vid#branch	current
Values:
 0	0.000000000000000e+00
	0.000000000000000e+00

*/